hiddenimports = [
    "pygubu.plugins.pygubu.accordionframe_bo",
    "pygubu.plugins.pygubu.calendarframe_bo",
    "pygubu.plugins.pygubu.colorinput_bo",
    "pygubu.plugins.pygubu.combobox_bo",
    "pygubu.plugins.pygubu.dialog_bo",
    "pygubu.plugins.pygubu.dockfw_bo",
    "pygubu.plugins.pygubu.editabletreeview_bo",
    "pygubu.plugins.pygubu.filterabletreeview_bo",
    "pygubu.plugins.pygubu.flodgauge_bo",
    "pygubu.plugins.pygubu.forms",
    "pygubu.plugins.pygubu.forms.pygubuwidget_bo",
    "pygubu.plugins.pygubu.forms.tkwidget_bo",
    "pygubu.plugins.pygubu.forms.ttkwidget_bo",
    "pygubu.plugins.pygubu.hideableframe_bo",
    "pygubu.plugins.pygubu.pathchooserinput_bo",
    "pygubu.plugins.pygubu.scrollbarhelper_bo",
    "pygubu.plugins.pygubu.scrolledframe_bo",
    "pygubu.plugins.pygubu.tkscrollbarhelper_bo",
    "pygubu.plugins.pygubu.tkscrolledframe_bo",
    "pygubu.plugins.tk.scrolledtext_bo",
    "pygubu.plugins.tk.tkstdwidgets",
    "pygubu.plugins.ttk.ttkstdwidgets",
]
